@@ -11,9 +11,9 @@
  *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *     GNU General Public License for more details.
  *
- *     You should have received a copy of the GNU General Public License
- *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *     You should have received a copy of the GNU General Public License along
+ *     with this program; if not, write to the Free Software Foundation, Inc.,
+ *     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  * This file is part of FoxEye's source: sockets library.
  */
@@ -47,6 +47,10 @@
 # include <idna.h>
 #endif
 
+#ifdef HAVE_SYS_IOCTL_H
+# include <sys/ioctl.h>
+#endif
+
 #ifndef SIGPOLL
 # define SIGPOLL SIGURG
 #endif
@@ -60,6 +64,11 @@
 /* solaris have this name as macro so drop it now */
 #undef sun
 
+/* OS X treats NI_MAXHOST as optional */
+#ifndef NI_MAXHOST
+# define NI_MAXHOST 1025
+#endif
+
 /*
  * Sequence:		socket.domain:	pollfd.fd:
  * unallocated		NULL		-2
